(function () {
  'use strict';

  function _defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  function _createClass(Constructor, protoProps, staticProps) {
    if (protoProps) _defineProperties(Constructor.prototype, protoProps);
    if (staticProps) _defineProperties(Constructor, staticProps);
    return Constructor;
  }

  function _inheritsLoose(subClass, superClass) {
    subClass.prototype = Object.create(superClass.prototype);
    subClass.prototype.constructor = subClass;

    _setPrototypeOf(subClass, superClass);
  }

  function _setPrototypeOf(o, p) {
    _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) {
      o.__proto__ = p;
      return o;
    };

    return _setPrototypeOf(o, p);
  }

  function isWindow(obj) {
    return obj != null && obj === obj.window;
  }

  function find(selector, context) {
    if (context === void 0) {
      context = null;
    }

    if (selector instanceof HTMLElement || selector instanceof Node || isWindow(selector)) {
      return [selector];
    } else if (selector instanceof NodeList) {
      return [].slice.call(selector);
    } else if (typeof selector === 'string') {
      var startElement = context ? find(context)[0] : document;
      return [].slice.call(startElement.querySelectorAll(selector));
    }

    return [];
  }

  function on(selector, event, cb, capture) {
    if (capture === void 0) {
      capture = false;
    }

    if (Array.isArray(selector)) {
      selector.forEach(function (item) {
        return on(item, event, cb, capture);
      });
      return;
    }

    var data = {
      cb: cb,
      capture: capture
    };

    if (!window._domassistevents) {
      window._domassistevents = {};
    }

    window._domassistevents["_" + event] = data;
    var el = find(selector);

    if (el.length) {
      el.forEach(function (item) {
        item.addEventListener(event, cb, capture);
      });
    }
  }

  function matches(el, selector) {
    var proto = Element.prototype;
    var match = false;
    var prefixes = ['matches', 'matchesSelector', 'webkitMatchesSelector', 'mozMatchesSelector', 'msMatchesSelector', 'oMatchesSelector'];
    prefixes.forEach(function (prefix) {
      if (proto.hasOwnProperty(prefix)) {
        match = proto[prefix];
      }
    });

    if (match) {
      return el ? match.call(el, selector) : null;
    }
  }

  function closest(el, selector) {
    do {
      if (matches(el, selector)) {
        return el;
      }

      el = el.parentElement || el.parentNode;
    } while (el !== null && el.nodeType === 1);

    return null;
  }

  function findOne(selector, el) {
    var found = find(selector, el);

    if (found.length) {
      return found[0];
    }

    return null;
  } // Check for the usage of native support for CustomEvents which is lacking
  // completely on IE.
  //


  function canIuseNativeCustom() {
    try {
      var p = new CustomEvent('t', {
        detail: {
          a: 'b'
        }
      });
      return p.type === 't' && p.detail.a === 'b';
    } catch (e) {
      return false;
    }
  } // Lousy polyfill for the Custom Event constructor for IE.


  var IECustomEvent = function CustomEvent(type, params) {
    var e = document.createEvent('CustomEvent');

    if (params) {
      e.initCustomEvent(type, params.bubbles, params.cancelable, params.detail);
    } else {
      e.initCustomEvent(type, false, false, undefined);
    }

    return e;
  };

  var DomassistCustomEvent = false;

  function fire(selector, type, params) {
    if (params === void 0) {
      params = {};
    }

    if (Array.isArray(selector)) {
      return selector.forEach(function (item) {
        return fire(item, type, params);
      });
    }

    if (!DomassistCustomEvent) {
      DomassistCustomEvent = canIuseNativeCustom() ? CustomEvent : IECustomEvent;
    }

    var els = find(selector);

    if (els.length) {
      if (params.bubbles !== false) {
        params.bubbles = true;
      }

      els.forEach(function (el) {
        var event = new DomassistCustomEvent(type, params);
        el.dispatchEvent(event);
      });
      return els;
    }
  }

  var setupReady = function setupReady(callbacks) {
    return function (callback) {
      callbacks.push(callback);

      function execute() {
        while (callbacks.length) {
          var fn = callbacks.shift();

          if (typeof fn === 'function') {
            fn();
          }
        }
      }

      function loaded() {
        document.removeEventListener('DOMContentLoaded', loaded);
        execute();
      }

      setTimeout(function () {
        if (document.readyState !== 'loading') {
          return execute();
        }
      }, 0);
      document.addEventListener('DOMContentLoaded', loaded);
    };
  };

  var ready = setupReady([]);

  /* global window */
  function attrobj(key, el) {
    var values = {};
    Object.keys(el.dataset).forEach(function (data) {
      if (data.match(new RegExp("^" + key)) && data !== key) {
        var optionName = data.replace(key, '');
        var isGlobal = false;

        if (optionName.match(/^Global/)) {
          optionName = optionName.replace('Global', '');
          isGlobal = true;
        }

        optionName = "" + optionName[0].toLowerCase() + optionName.slice(1);

        if (isGlobal) {
          values[optionName] = window[el.dataset[data]];
        } else {
          values[optionName] = el.dataset[data];
        }

        if (typeof values[optionName] === 'undefined' || values[optionName] === '') {
          values[optionName] = true;
        }
      }
    });
    return values;
  }

  var l = function l() {
    var l = [].slice.call(arguments);
    return l.unshift(!1), t.apply(null, l);
  };

  function t() {
    var t = [].slice.call(arguments),
        e = t[0],
        n = {};
    return t.slice(1, t.length).forEach(function (r) {
      Object.keys(r).forEach(function (a) {
        if (!e || void 0 !== t[1][a]) {
          var c = r[a];
          n[a] = Array.isArray(c) || "object" != typeof c || "object" != typeof n[a] ? c : (e ? l.defaults : l)(n[a], c);
        }
      });
    }), n;
  }

  l.defaults = function () {
    var l = [].slice.call(arguments);
    return l.unshift(!0), t.apply(null, l);
  };

  function s(t) {
    return t.parentNode ? JSON.parse(JSON.stringify(t.parentNode.dataset)).module ? t.parentNode : s(t.parentNode) : t;
  }

  var r = {
    find: find,
    findOne: findOne,
    on: on
  },
      d = function () {
    function t(e, o) {
      return void 0 === o && (o = ""), this.log("begin setup"), this.el = e, this.els = {}, this.options = l({}, this.defaults, attrobj("module", this.el)), this.moduleName = o || this.el.dataset.module, this.setUps = {
        actions: [],
        named: [],
        options: []
      }, this.boundActionRouter = this.actionRouter.bind(this), this.preInit(), this.storeRef(), this.setupActions(), this.setupNamed(), this.verifyRequired(), this.postInit(), this.log("initalized"), t.debug && (this.el.module = this), this;
    }

    var e,
        o = t.prototype;
    return o.preInit = function () {}, o.postInit = function () {}, o.verifyRequired = function () {
      var t = this;
      return this.required === {} || (void 0 !== this.required.options && (this.setUps.options = Object.keys(this.options)), Object.keys(this.required).forEach(function (e) {
        t.required[e].forEach(function (o) {
          if (t.setUps[e].indexOf(o) < 0) throw new Error(o + " is required as " + e + " for " + t.moduleName + ", but is missing!");
        });
      })), this;
    }, o.setupActions = function () {
      var t = this;
      this.setupAction(this.el), this.find("[data-action]").forEach(function (e) {
        s(e) === t.el && t.setupAction(e);
      });
    }, o.setupAction = function (e) {
      if ("true" !== e.dataset.domoduleActionProcessed) {
        var o = t.parseAction(e),
            i = o.name,
            n = o.type;
        i && ("function" == typeof this[i] ? (this.log(i + " bound"), this.storeSetUp(i, "actions"), r.on(e, n, this.boundActionRouter), e.dataset.domoduleActionProcessed = "true") : this.log(i + " was registered, but there is no function set up"));
      }
    }, o.actionRouter = function (e) {
      var o = e.currentTarget,
          n = t.parseAction(o).name,
          s = attrobj("action", o);
      this[n].call(this, o, e, s);
    }, o.setupNamed = function () {
      var t = this;
      this.find("[data-name]").forEach(function (e) {
        s(e) === t.el && (e.dataset.domoduleNameProcessed || (t.els[e.dataset.name] = e, t.storeSetUp(e.dataset.name, "named"), e.dataset.domoduleNameProcessed = "true", e.dataset.domoduleOwner = t.id));
      });
    }, o.storeRef = function () {
      if (void 0 === window.domorefs && (window.domorefs = {}), void 0 !== window.domorefs[this.el.dataset.moduleUid]) return !1;
      this.id = this.uuid, this.el.dataset.moduleUid = this.id, window.domorefs[this.el.dataset.moduleUid] = this;
    }, o.find = function (t) {
      return r.find(t, this.el);
    }, o.findOne = function (t) {
      return r.findOne(t, this.el);
    }, o.findByName = function (t) {
      return this.els[t];
    }, o.getOption = function (t) {
      return this.options[t];
    }, o.storeSetUp = function (t, e) {
      this.setUps[e].indexOf(t) < 0 && this.setUps[e].push(t);
    }, o.destroy = function () {
      var e = this;
      r.find("[data-action]", this.el.parentNode).forEach(function (o) {
        if ("true" === o.dataset.domoduleActionProcessed) {
          var i = t.parseAction(o);
          o.removeEventListener(i.type, e.boundActionRouter), o.dataset.domoduleActionProcessed = "false";
        }
      });
    }, t.parseAction = function (t) {
      var e = t.dataset,
          o = e.actionType;
      return {
        name: e.action,
        type: void 0 === o ? "click" : o
      };
    }, t.getInstance = function (t) {
      if (t instanceof Node) return window.domorefs[t.dataset.moduleUid];
      throw new Error("getInstance expects a dom node");
    }, t.register = function (e, o) {
      "function" == typeof e && (e = (o = e).prototype.constructor.name), window.domodules || (window.domodules = {}), t.log("Registering " + e), window.domodules[e] = o;
    }, t.discover = function (e) {
      if (void 0 === e && (e = "body"), t.log("Discovering modules..."), window.domodules) {
        var o;
        o = e instanceof Node ? [e] : Array.isArray(e) ? e : r.find(e);
        var i = [];
        return o.forEach(function (e) {
          r.find("[data-module]", e).forEach(function (e) {
            var o = e.dataset.module;

            if (o && "function" == typeof window.domodules[o]) {
              if ("object" == typeof window.domorefs && void 0 !== window.domorefs[e.dataset.moduleUid]) return;
              t.log(o + " found"), i.push(new window.domodules[o](e));
            }
          });
        }), i;
      }

      t.log("No modules found");
    }, o.log = function (e) {
      t.log(this.constructor.name + ": " + e);
    }, t.log = function (e) {
      t.debug && console.log("[DOMODULE] " + e);
    }, (e = [{
      key: "required",
      get: function get() {
        return {};
      }
    }, {
      key: "defaults",
      get: function get() {
        return {};
      }
    }, {
      key: "uuid",
      get: function get() {
        return "xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx".replace(/[xy]/g, function (t) {
          var e = 16 * Math.random() | 0;
          return ("x" === t ? e : 3 & e | 8).toString(16);
        });
      }
    }]) && function (t, e) {
      for (var o = 0; o < e.length; o++) {
        var i = e[o];
        i.enumerable = i.enumerable || !1, i.configurable = !0, "value" in i && (i.writable = !0), Object.defineProperty(t, i.key, i);
      }
    }(t.prototype, e), t;
  }();

  d.debug = "object" == typeof window.localStorage && window.localStorage.getItem("DomoduleDebug"), d.autoDiscover = !0, window.addEventListener("DOMContentLoaded", function () {
    d.autoDiscover && d.discover();
  });

  var debounce = function debounce(func, wait, immediate) {
    var timeout;
    return function () {
      var context = this;
      var args = arguments; // eslint-disable-line prefer-rest-params

      var later = function later() {
        timeout = null;

        if (!immediate) {
          func.apply(context, args);
        }
      };

      var callNow = immediate && !timeout;
      clearTimeout(timeout);
      timeout = setTimeout(later, wait);

      if (callNow) {
        func.apply(context, args);
      }
    };
  };

  /* eslint-env browser */

  var scrollHandlers = [];
  on(window, 'scroll', function (event) {
    scrollHandlers.forEach(function (listener) {
      return listener.handler.call(null, event);
    });
  });
  var index = {
    on: function on(callback, wait, immediate) {
      var handler = callback;

      if (wait) {
        handler = debounce(callback, wait, immediate);
      }

      scrollHandlers.push({
        handler: handler,
        callback: callback,
        wait: wait,
        immediate: immediate
      });
    },
    off: function off(callback, wait, immediate) {
      if (!callback) {
        scrollHandlers.length = 0;
      } else {
        scrollHandlers = scrollHandlers.filter(function (listener) {
          return listener.callback !== callback && listener.wait !== wait && listener.immediate !== immediate;
        });
      }
    }
  };

  /* eslint-disable no-restricted-syntax, guard-for-in, prefer-rest-params, no-use-before-define */
  var aug = function aug() {
    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    args.unshift(false);
    return merge.apply(null, args);
  };

  aug.defaults = function () {
    for (var _len2 = arguments.length, args = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
      args[_key2] = arguments[_key2];
    }

    args.unshift(true);
    return merge.apply(null, args);
  }; // first arg is 'true' if using defaults-only version
  // first arg is 'false' if just doing deep merge:


  function merge() {
    for (var _len3 = arguments.length, args = new Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
      args[_key3] = arguments[_key3];
    }

    var useDefaults = args[0];
    var destObject = {}; // for each object in the rest of the argument list:

    args.slice(1, args.length).forEach(function (prop) {
      // for each property in the current object:
      for (var propName in prop) {
        // default-only mode skips the property if it's not present in the first object
        if (useDefaults && args[1][propName] === undefined) {
          return;
        } // get the value we are going to assign to the dest object:


        var propValue = prop[propName]; // if that value is an array just assign it to the dest object:

        if (Array.isArray(propValue)) {
          destObject[propName] = propValue;
          continue;
        } // if the source and destination values are both objects then recursively merge them:


        if (typeof propValue === 'object' && typeof destObject[propName] === 'object') {
          // get the right merging function for the recursive merge:
          var merger = useDefaults ? aug.defaults : aug;
          destObject[propName] = merger(destObject[propName], propValue);
          continue;
        } // otherwise just assign the value to the destination object:


        destObject[propName] = propValue;
      }
    });
    return destObject;
  }

  /* eslint-disable callback-return */

  var GATrack = {
    sendEvent: function sendEvent(category, action, label, callback, timeout) {
      if (callback === void 0) {
        callback = null;
      }

      if (timeout === void 0) {
        timeout = 1000;
      }

      if (GATrack.prefix) {
        category = GATrack.prefix + "-" + category;
      }

      var onHit = function onHit() {};

      GATrack.log(category, action, label);

      if (!GATrack.isEnabled) {
        if (typeof callback === 'function') {
          callback();
        }

        return GATrack;
      }

      if (typeof callback === 'function') {
        var timeoutId;

        onHit = function onHit() {
          if (!timeoutId) {
            return;
          }

          clearTimeout(timeoutId);
          timeoutId = null;
          callback();
        };

        timeoutId = setTimeout(onHit, timeout);
      }

      if (GATrack.isGAQ) {
        _gaq.push(['_trackEvent', category, action, label, null, false]);

        _gaq.push(onHit);
      } else if (GATrack.isGTag) {
        // Gtag check needs to go before since gtag creates a ga variable
        var payload = {
          event_category: category,
          event_label: label,
          event_callback: onHit
        };
        GATrack.sendData('event', action, payload);
      } else if (GATrack.isGA) {
        var options = {
          transport: 'beacon',
          hitCallback: onHit
        };
        GATrack.sendData('send', 'event', category, action, label, options);
      }
    },
    send: function send() {
      // eslint-disable-next-line prefer-rest-params
      var args = Array.prototype.slice.call(arguments);

      if (GATrack.isGA) {
        args.unshift('send');
      }

      GATrack.sendData.apply(null, args);
    },
    sendData: function sendData() {
      if (!GATrack.isEnabled) {
        return GATrack;
      } // eslint-disable-next-line prefer-rest-params


      var args = Array.prototype.slice.call(arguments);

      if (GATrack.isGA) {
        if (GATrack.trackerName) {
          args[0] = GATrack.trackerName + "." + args[0];
        }

        ga.apply(null, args);
      } else if (GATrack.isGTag) {
        gtag.apply(null, args);
      }
    },
    getData: function getData(element, options) {
      if (options === void 0) {
        options = {};
      }

      var href = element.dataset.gaTrackHref || element.getAttribute('href');
      var category = element.dataset.gaTrack || options.category || 'ga-track';
      var label = element.dataset.gaTrackLabel || options.label || href;
      var action = element.dataset.gaTrackAction || options.action || element.textContent.trim();
      return {
        href: href,
        category: category,
        label: label,
        action: action
      };
    },
    track: function track(element, options) {
      if (options === void 0) {
        options = {};
      }

      if (Array.isArray(element)) {
        element.forEach(function (data) {
          find(data.element).forEach(function (el) {
            GATrack.track(el, data);
          });
        });
        return;
      }

      if (typeof element.dataset.gaTrackInitialised !== 'undefined') {
        return;
      }

      element.dataset.gaTrackInitialised = 'true';
      options = aug({}, GATrack.defaults, options);
      this.log('tracking', element, options);
      on(element, 'click', function (event) {
        GATrack.onTrackedClick(element, event, options);
      });
    },
    onTrackedClick: function onTrackedClick(element, event, options) {
      var data = GATrack.getData(element, options);
      var target = element.getAttribute('target');
      var callback = null;

      if (element.dataset.gaTrackHref === 'false') {
        event.preventDefault();
      } else if (data.href && !event.metaKey && event.which === 1 && target !== '_blank') {
        event.preventDefault();

        callback = function callback() {
          window.location = data.href;
        };
      }

      GATrack.sendEvent(data.category, data.action, data.label, callback, options.timeout);
    },
    autotrack: function autotrack() {
      if (GATrack.autotracking === true) {
        return;
      }

      GATrack.autotracking = true;
      var selector = '[data-ga-track]';
      on(document.body, 'click', function (event) {
        var element = event.target;

        if (!matches(element, selector)) {
          element = closest(element, selector);
        }

        if (element) {
          GATrack.onTrackedClick(element, event, GATrack.defaults);
        }
      });
    },
    log: function log() {
      if (GATrack.debug) {
        var _console;

        for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
          args[_key] = arguments[_key];
        }

        (_console = console).log.apply(_console, ['GATRACK'].concat(args)); //eslint-disable-line no-console

      }
    },
    isNullOrEnforced: function isNullOrEnforced(provider) {
      return GATrack.force === null || GATrack.force === provider;
    },

    get isGAQ() {
      // eslint-disable-line no-underscore-dangle
      return typeof window._gaq !== 'undefined' && GATrack.isNullOrEnforced('gaq');
    },

    get isGTag() {
      return typeof window.gtag !== 'undefined' && GATrack.isNullOrEnforced('gtag');
    },

    get isGA() {
      return typeof window.ga !== 'undefined' && GATrack.isNullOrEnforced('ga');
    },

    get isEnabled() {
      return GATrack.isGA || GATrack.isGTag || GATrack.isGAQ;
    },

    debug: typeof window.localStorage === 'object' && window.localStorage.getItem('GATrackDebug'),
    prefix: null,
    trackerName: '',
    force: null,
    defaults: {
      timeout: 1000
    }
  };

  var outlineTracked = function outlineTracked() {
    return find('[data-ga-track-initialised], [data-ga-track]').map(function (el) {
      el.style.outline = 'red dotted 1px';
      return el;
    });
  };

  GATrack.debug = window.localStorage.getItem('GATrackDebug');
  var outline = window.localStorage.getItem('GATrackOutline');
  window.GAOutlineTracked = outlineTracked;
  ready(function () {
    if (!window._GATrack_) {
      window._GATrack_ = GATrack;

      if (typeof window.gaTrackerName_ !== 'undefined') {
        GATrack.trackerName = window.gaTrackerName_;
      }

      GATrack.autotrack();

      if (outline) {
        outlineTracked();
      }
    }
  });

  var scrollPercent = 0;
  var wHeight = 0;
  var dHeight = 0;
  var trackLength = 0;
  var location = document.location.toString();
  var hasScrolled = false;
  var cache = [25, 50, 75, 100];
  var Events = {
    Scroll: 'user:scroll'
  };

  function getDocHeight() {
    return Math.max(document.body.scrollHeight, document.documentElement.scrollHeight, document.body.offsetHeight, document.documentElement.offsetHeight, document.body.clientHeight, document.documentElement.clientHeight);
  }

  function computeSizes() {
    wHeight = window.innerHeight || (document.documentElement || document.body).clientHeight;
    dHeight = getDocHeight();
    trackLength = dHeight - wHeight;
  }

  function getScrolledAmount() {
    var scrollTop = window.pageYOffset || (document.documentElement || document.body.parentNode || document.body).scrollTop;
    return Math.floor(scrollTop / trackLength * 100);
  }

  function trackScrollPosition(amount, overriddenLabel) {
    var category = 'scroll';
    var action = location;
    var label = overriddenLabel || "Scrolled " + amount + "%";

    if (GATrack.isGTag) {
      // Gtag check needs to go before since gtag creates a ga variable
      var payload = {
        event_category: category,
        event_label: label,
        value: amount
      };
      GATrack.sendData('event', action, payload);
    } else {
      GATrack.sendData('send', 'event', category, action, label, amount);
    }

    fire(document.body, Events.Scroll, {
      detail: {
        amount: amount
      }
    });
  }

  function scrollCheck() {
    if (!cache.length) {
      return;
    }

    scrollPercent = getScrolledAmount();

    if (!hasScrolled && scrollPercent > 0) {
      hasScrolled = true;
      GATrack.sendEvent('scroll', location, 'Scrolled');
    }

    var events = cache.filter(function (a) {
      return a <= scrollPercent;
    });
    events.forEach(function (e) {
      trackScrollPosition(e);
      cache.splice(cache.indexOf(e), 1);
    });
  }

  ready(function () {
    if (window.disableGATrackScroll !== true) {
      computeSizes();
      index.on(scrollCheck);
      window.addEventListener('resize', computeSizes, false);
    }
  });

  function percentage(percent, total) {
    return Math.round(percent / 100 * total);
  }

  var supportsPassive = false;

  try {
    var opts = Object.defineProperty({}, 'passive', {
      get: function get() {
        // eslint-disable-next-line no-unused-vars
        supportsPassive = true;
      }
    });
    window.addEventListener('testPassive', null, opts);
    window.removeEventListener('testPassive', null, opts);
  } catch (e) {// Silently erroring
  }

  var InfiniteScrollTracker = /*#__PURE__*/function (_Domodule) {
    _inheritsLoose(InfiniteScrollTracker, _Domodule);

    function InfiniteScrollTracker() {
      return _Domodule.apply(this, arguments) || this;
    }

    var _proto = InfiniteScrollTracker.prototype;

    _proto.postInit = function postInit() {
      var scrollOption = supportsPassive ? {
        passive: true
      } : false;
      this.observer = new MutationObserver(this.onMutationHappen.bind(this));
      this.observer.observe(this.el, {
        childList: true
      });
      this.tracked = [];
      this.pending = [];
      this.getElements();
      window.addEventListener('resize', this.computeSizes.bind(this), false);
      window.addEventListener('scroll', this.onScroll.bind(this), scrollOption);
    };

    _proto.getElements = function getElements() {
      var _this = this;

      var elements = find(this.options.selector);
      var currentScrollPosition = InfiniteScrollTracker.getScrollPosition();
      elements.forEach(function (element, i) {
        var trackedConf = _this.tracked[i] || {};

        if (!trackedConf.id) {
          trackedConf.id = element.id || _this.generateId();
          element.id = trackedConf.id;
          trackedConf.positions = _this.getScrollPositions(element, i, currentScrollPosition);
          trackedConf.element = element;
        }

        _this.tracked[i] = trackedConf;
      });
      this.refreshPending();
    };

    _proto.refreshPending = function refreshPending() {
      var _this2 = this;

      this.pending = [];
      this.tracked.forEach(function (el) {
        el.positions.forEach(function (pos) {
          if (!pos.triggered) {
            _this2.pending.push(pos);
          }
        });
      });
    };

    _proto.onMutationHappen = function onMutationHappen() {
      this.getElements();
    };

    _proto.onScroll = function onScroll() {
      if (this.pending.length) {
        var currentPosition = InfiniteScrollTracker.getScrollPosition();
        var triggered = [];

        for (var i = 0; i < this.pending.length; i++) {
          if (this.pending[i].scroll <= currentPosition) {
            triggered.push(this.pending[i]);
            this.pending[i].triggered = true;
          } else {
            break;
          }
        }

        if (triggered.length) {
          this.refreshPending();
          triggered.forEach(function (trigger) {
            return trackScrollPosition(trigger.amount, trigger.label);
          });
        }
      }
    };

    _proto.computeSizes = function computeSizes() {
      var _this3 = this;

      var currentPosition = InfiniteScrollTracker.getScrollPosition();
      this.tracked.forEach(function (el, i) {
        el.positions = _this3.getScrollPositions(el.element, i, currentPosition, el.pending);
      });
      this.refreshPending();
    };

    _proto.generateId = function generateId() {
      return this.options.selector.replace('.', '') + "-" + Date.now();
    };

    _proto.getScrollPositions = function getScrollPositions(element, elementPosition, currentScroll, pending) {
      var positions = [0, 25, 50, 75, 100];
      var trackablePositions = Array.isArray(pending) ? pending.filter(function (p) {
        return !p.triggered;
      }).map(function (p) {
        return p.amount;
      }) : positions;
      var labelText = elementPosition === 0 ? this.options.landingLabel : this.options.extraLabel + " " + (elementPosition + 1);

      if (trackablePositions.length) {
        var rect = element.getBoundingClientRect();
        var top = rect.top + currentScroll;
        return positions.map(function (amount, i) {
          if (positions[i] === amount) {
            var label;

            if (i === 0) {
              label = "Scrolled " + labelText;
            } else {
              label = "Scrolled " + labelText + " - " + amount + "%";
            }

            return {
              amount: amount,
              triggered: false,
              label: label,
              scroll: top + percentage(amount, rect.height)
            };
          }

          return pending[i];
        });
      }
    };

    InfiniteScrollTracker.getScrollPosition = function getScrollPosition() {
      return (window.pageYOffset || document.scrollTop) - (document.clientTop || 0) || 0;
    };

    _createClass(InfiniteScrollTracker, [{
      key: "defaults",
      get: function get() {
        return {
          landingLabel: 'Landing Article',
          extraLabel: 'Article'
        };
      }
    }, {
      key: "required",
      get: function get() {
        return {
          options: ['selector']
        };
      }
    }]);

    return InfiniteScrollTracker;
  }(d);

  window.disableGATrackScroll = true;
  d.register('InfiniteScrollTracker', InfiniteScrollTracker);

}());
//# sourceMappingURL=ga-track-scroll-module.bundle.js.map
