import Domodule from 'domodule';
import { ready, fire, find } from 'domassist';
import ScrollBus from 'scroll-bus';
import GATrack from 'ga-track';

function _defineProperties(target, props) {
  for (var i = 0; i < props.length; i++) {
    var descriptor = props[i];
    descriptor.enumerable = descriptor.enumerable || false;
    descriptor.configurable = true;
    if ("value" in descriptor) descriptor.writable = true;
    Object.defineProperty(target, descriptor.key, descriptor);
  }
}

function _createClass(Constructor, protoProps, staticProps) {
  if (protoProps) _defineProperties(Constructor.prototype, protoProps);
  if (staticProps) _defineProperties(Constructor, staticProps);
  return Constructor;
}

function _inheritsLoose(subClass, superClass) {
  subClass.prototype = Object.create(superClass.prototype);
  subClass.prototype.constructor = subClass;

  _setPrototypeOf(subClass, superClass);
}

function _setPrototypeOf(o, p) {
  _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) {
    o.__proto__ = p;
    return o;
  };

  return _setPrototypeOf(o, p);
}

var scrollPercent = 0;
var wHeight = 0;
var dHeight = 0;
var trackLength = 0;
var location = document.location.toString();
var hasScrolled = false;
var cache = [25, 50, 75, 100];
var Events = {
  Scroll: 'user:scroll'
};

function getDocHeight() {
  return Math.max(document.body.scrollHeight, document.documentElement.scrollHeight, document.body.offsetHeight, document.documentElement.offsetHeight, document.body.clientHeight, document.documentElement.clientHeight);
}

function computeSizes() {
  wHeight = window.innerHeight || (document.documentElement || document.body).clientHeight;
  dHeight = getDocHeight();
  trackLength = dHeight - wHeight;
}

function getScrolledAmount() {
  var scrollTop = window.pageYOffset || (document.documentElement || document.body.parentNode || document.body).scrollTop;
  return Math.floor(scrollTop / trackLength * 100);
}

function trackScrollPosition(amount, overriddenLabel) {
  var category = 'scroll';
  var action = location;
  var label = overriddenLabel || "Scrolled " + amount + "%";

  if (GATrack.isGTag) {
    // Gtag check needs to go before since gtag creates a ga variable
    var payload = {
      event_category: category,
      event_label: label,
      value: amount
    };
    GATrack.sendData('event', action, payload);
  } else {
    GATrack.sendData('send', 'event', category, action, label, amount);
  }

  fire(document.body, Events.Scroll, {
    detail: {
      amount: amount
    }
  });
}

function scrollCheck() {
  if (!cache.length) {
    return;
  }

  scrollPercent = getScrolledAmount();

  if (!hasScrolled && scrollPercent > 0) {
    hasScrolled = true;
    GATrack.sendEvent('scroll', location, 'Scrolled');
  }

  var events = cache.filter(function (a) {
    return a <= scrollPercent;
  });
  events.forEach(function (e) {
    trackScrollPosition(e);
    cache.splice(cache.indexOf(e), 1);
  });
}

ready(function () {
  if (window.disableGATrackScroll !== true) {
    computeSizes();
    ScrollBus.on(scrollCheck);
    window.addEventListener('resize', computeSizes, false);
  }
});

function percentage(percent, total) {
  return Math.round(percent / 100 * total);
}

var supportsPassive = false;

try {
  var opts = Object.defineProperty({}, 'passive', {
    get: function get() {
      // eslint-disable-next-line no-unused-vars
      supportsPassive = true;
    }
  });
  window.addEventListener('testPassive', null, opts);
  window.removeEventListener('testPassive', null, opts);
} catch (e) {// Silently erroring
}

var InfiniteScrollTracker = /*#__PURE__*/function (_Domodule) {
  _inheritsLoose(InfiniteScrollTracker, _Domodule);

  function InfiniteScrollTracker() {
    return _Domodule.apply(this, arguments) || this;
  }

  var _proto = InfiniteScrollTracker.prototype;

  _proto.postInit = function postInit() {
    var scrollOption = supportsPassive ? {
      passive: true
    } : false;
    this.observer = new MutationObserver(this.onMutationHappen.bind(this));
    this.observer.observe(this.el, {
      childList: true
    });
    this.tracked = [];
    this.pending = [];
    this.getElements();
    window.addEventListener('resize', this.computeSizes.bind(this), false);
    window.addEventListener('scroll', this.onScroll.bind(this), scrollOption);
  };

  _proto.getElements = function getElements() {
    var _this = this;

    var elements = find(this.options.selector);
    var currentScrollPosition = InfiniteScrollTracker.getScrollPosition();
    elements.forEach(function (element, i) {
      var trackedConf = _this.tracked[i] || {};

      if (!trackedConf.id) {
        trackedConf.id = element.id || _this.generateId();
        element.id = trackedConf.id;
        trackedConf.positions = _this.getScrollPositions(element, i, currentScrollPosition);
        trackedConf.element = element;
      }

      _this.tracked[i] = trackedConf;
    });
    this.refreshPending();
  };

  _proto.refreshPending = function refreshPending() {
    var _this2 = this;

    this.pending = [];
    this.tracked.forEach(function (el) {
      el.positions.forEach(function (pos) {
        if (!pos.triggered) {
          _this2.pending.push(pos);
        }
      });
    });
  };

  _proto.onMutationHappen = function onMutationHappen() {
    this.getElements();
  };

  _proto.onScroll = function onScroll() {
    if (this.pending.length) {
      var currentPosition = InfiniteScrollTracker.getScrollPosition();
      var triggered = [];

      for (var i = 0; i < this.pending.length; i++) {
        if (this.pending[i].scroll <= currentPosition) {
          triggered.push(this.pending[i]);
          this.pending[i].triggered = true;
        } else {
          break;
        }
      }

      if (triggered.length) {
        this.refreshPending();
        triggered.forEach(function (trigger) {
          return trackScrollPosition(trigger.amount, trigger.label);
        });
      }
    }
  };

  _proto.computeSizes = function computeSizes() {
    var _this3 = this;

    var currentPosition = InfiniteScrollTracker.getScrollPosition();
    this.tracked.forEach(function (el, i) {
      el.positions = _this3.getScrollPositions(el.element, i, currentPosition, el.pending);
    });
    this.refreshPending();
  };

  _proto.generateId = function generateId() {
    return this.options.selector.replace('.', '') + "-" + Date.now();
  };

  _proto.getScrollPositions = function getScrollPositions(element, elementPosition, currentScroll, pending) {
    var positions = [0, 25, 50, 75, 100];
    var trackablePositions = Array.isArray(pending) ? pending.filter(function (p) {
      return !p.triggered;
    }).map(function (p) {
      return p.amount;
    }) : positions;
    var labelText = elementPosition === 0 ? this.options.landingLabel : this.options.extraLabel + " " + (elementPosition + 1);

    if (trackablePositions.length) {
      var rect = element.getBoundingClientRect();
      var top = rect.top + currentScroll;
      return positions.map(function (amount, i) {
        if (positions[i] === amount) {
          var label;

          if (i === 0) {
            label = "Scrolled " + labelText;
          } else {
            label = "Scrolled " + labelText + " - " + amount + "%";
          }

          return {
            amount: amount,
            triggered: false,
            label: label,
            scroll: top + percentage(amount, rect.height)
          };
        }

        return pending[i];
      });
    }
  };

  InfiniteScrollTracker.getScrollPosition = function getScrollPosition() {
    return (window.pageYOffset || document.scrollTop) - (document.clientTop || 0) || 0;
  };

  _createClass(InfiniteScrollTracker, [{
    key: "defaults",
    get: function get() {
      return {
        landingLabel: 'Landing Article',
        extraLabel: 'Article'
      };
    }
  }, {
    key: "required",
    get: function get() {
      return {
        options: ['selector']
      };
    }
  }]);

  return InfiniteScrollTracker;
}(Domodule);

window.disableGATrackScroll = true;
Domodule.register('InfiniteScrollTracker', InfiniteScrollTracker);
//# sourceMappingURL=ga-track-scroll-module.js.map
