var gaTrackScroll = (function (exports) {
  'use strict';

  function isWindow(obj) {
    return obj != null && obj === obj.window;
  }

  function find(selector, context) {
    if (context === void 0) {
      context = null;
    }

    if (selector instanceof HTMLElement || selector instanceof Node || isWindow(selector)) {
      return [selector];
    } else if (selector instanceof NodeList) {
      return [].slice.call(selector);
    } else if (typeof selector === 'string') {
      var startElement = context ? find(context)[0] : document;
      return [].slice.call(startElement.querySelectorAll(selector));
    }

    return [];
  }

  function on(selector, event, cb, capture) {
    if (capture === void 0) {
      capture = false;
    }

    if (Array.isArray(selector)) {
      selector.forEach(function (item) {
        return on(item, event, cb, capture);
      });
      return;
    }

    var data = {
      cb: cb,
      capture: capture
    };

    if (!window._domassistevents) {
      window._domassistevents = {};
    }

    window._domassistevents["_" + event] = data;
    var el = find(selector);

    if (el.length) {
      el.forEach(function (item) {
        item.addEventListener(event, cb, capture);
      });
    }
  }

  function matches(el, selector) {
    var proto = Element.prototype;
    var match = false;
    var prefixes = ['matches', 'matchesSelector', 'webkitMatchesSelector', 'mozMatchesSelector', 'msMatchesSelector', 'oMatchesSelector'];
    prefixes.forEach(function (prefix) {
      if (proto.hasOwnProperty(prefix)) {
        match = proto[prefix];
      }
    });

    if (match) {
      return el ? match.call(el, selector) : null;
    }
  }

  function closest(el, selector) {
    do {
      if (matches(el, selector)) {
        return el;
      }

      el = el.parentElement || el.parentNode;
    } while (el !== null && el.nodeType === 1);

    return null;
  }
  // completely on IE.
  //


  function canIuseNativeCustom() {
    try {
      var p = new CustomEvent('t', {
        detail: {
          a: 'b'
        }
      });
      return p.type === 't' && p.detail.a === 'b';
    } catch (e) {
      return false;
    }
  } // Lousy polyfill for the Custom Event constructor for IE.


  var IECustomEvent = function CustomEvent(type, params) {
    var e = document.createEvent('CustomEvent');

    if (params) {
      e.initCustomEvent(type, params.bubbles, params.cancelable, params.detail);
    } else {
      e.initCustomEvent(type, false, false, undefined);
    }

    return e;
  };

  var DomassistCustomEvent = false;

  function fire(selector, type, params) {
    if (params === void 0) {
      params = {};
    }

    if (Array.isArray(selector)) {
      return selector.forEach(function (item) {
        return fire(item, type, params);
      });
    }

    if (!DomassistCustomEvent) {
      DomassistCustomEvent = canIuseNativeCustom() ? CustomEvent : IECustomEvent;
    }

    var els = find(selector);

    if (els.length) {
      if (params.bubbles !== false) {
        params.bubbles = true;
      }

      els.forEach(function (el) {
        var event = new DomassistCustomEvent(type, params);
        el.dispatchEvent(event);
      });
      return els;
    }
  }

  var setupReady = function setupReady(callbacks) {
    return function (callback) {
      callbacks.push(callback);

      function execute() {
        while (callbacks.length) {
          var fn = callbacks.shift();

          if (typeof fn === 'function') {
            fn();
          }
        }
      }

      function loaded() {
        document.removeEventListener('DOMContentLoaded', loaded);
        execute();
      }

      setTimeout(function () {
        if (document.readyState !== 'loading') {
          return execute();
        }
      }, 0);
      document.addEventListener('DOMContentLoaded', loaded);
    };
  };

  var ready = setupReady([]);

  var debounce = function debounce(func, wait, immediate) {
    var timeout;
    return function () {
      var context = this;
      var args = arguments; // eslint-disable-line prefer-rest-params

      var later = function later() {
        timeout = null;

        if (!immediate) {
          func.apply(context, args);
        }
      };

      var callNow = immediate && !timeout;
      clearTimeout(timeout);
      timeout = setTimeout(later, wait);

      if (callNow) {
        func.apply(context, args);
      }
    };
  };

  /* eslint-env browser */

  var scrollHandlers = [];
  on(window, 'scroll', function (event) {
    scrollHandlers.forEach(function (listener) {
      return listener.handler.call(null, event);
    });
  });
  var index = {
    on: function on(callback, wait, immediate) {
      var handler = callback;

      if (wait) {
        handler = debounce(callback, wait, immediate);
      }

      scrollHandlers.push({
        handler: handler,
        callback: callback,
        wait: wait,
        immediate: immediate
      });
    },
    off: function off(callback, wait, immediate) {
      if (!callback) {
        scrollHandlers.length = 0;
      } else {
        scrollHandlers = scrollHandlers.filter(function (listener) {
          return listener.callback !== callback && listener.wait !== wait && listener.immediate !== immediate;
        });
      }
    }
  };

  /* eslint-disable no-restricted-syntax, guard-for-in, prefer-rest-params, no-use-before-define */
  var aug = function aug() {
    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    args.unshift(false);
    return merge.apply(null, args);
  };

  aug.defaults = function () {
    for (var _len2 = arguments.length, args = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
      args[_key2] = arguments[_key2];
    }

    args.unshift(true);
    return merge.apply(null, args);
  }; // first arg is 'true' if using defaults-only version
  // first arg is 'false' if just doing deep merge:


  function merge() {
    for (var _len3 = arguments.length, args = new Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
      args[_key3] = arguments[_key3];
    }

    var useDefaults = args[0];
    var destObject = {}; // for each object in the rest of the argument list:

    args.slice(1, args.length).forEach(function (prop) {
      // for each property in the current object:
      for (var propName in prop) {
        // default-only mode skips the property if it's not present in the first object
        if (useDefaults && args[1][propName] === undefined) {
          return;
        } // get the value we are going to assign to the dest object:


        var propValue = prop[propName]; // if that value is an array just assign it to the dest object:

        if (Array.isArray(propValue)) {
          destObject[propName] = propValue;
          continue;
        } // if the source and destination values are both objects then recursively merge them:


        if (typeof propValue === 'object' && typeof destObject[propName] === 'object') {
          // get the right merging function for the recursive merge:
          var merger = useDefaults ? aug.defaults : aug;
          destObject[propName] = merger(destObject[propName], propValue);
          continue;
        } // otherwise just assign the value to the destination object:


        destObject[propName] = propValue;
      }
    });
    return destObject;
  }

  /* eslint-disable callback-return */

  var GATrack = {
    sendEvent: function sendEvent(category, action, label, callback, timeout) {
      if (callback === void 0) {
        callback = null;
      }

      if (timeout === void 0) {
        timeout = 1000;
      }

      if (GATrack.prefix) {
        category = GATrack.prefix + "-" + category;
      }

      var onHit = function onHit() {};

      GATrack.log(category, action, label);

      if (!GATrack.isEnabled) {
        if (typeof callback === 'function') {
          callback();
        }

        return GATrack;
      }

      if (typeof callback === 'function') {
        var timeoutId;

        onHit = function onHit() {
          if (!timeoutId) {
            return;
          }

          clearTimeout(timeoutId);
          timeoutId = null;
          callback();
        };

        timeoutId = setTimeout(onHit, timeout);
      }

      if (GATrack.isGAQ) {
        _gaq.push(['_trackEvent', category, action, label, null, false]);

        _gaq.push(onHit);
      } else if (GATrack.isGTag) {
        // Gtag check needs to go before since gtag creates a ga variable
        var payload = {
          event_category: category,
          event_label: label,
          event_callback: onHit
        };
        GATrack.sendData('event', action, payload);
      } else if (GATrack.isGA) {
        var options = {
          transport: 'beacon',
          hitCallback: onHit
        };
        GATrack.sendData('send', 'event', category, action, label, options);
      }
    },
    send: function send() {
      // eslint-disable-next-line prefer-rest-params
      var args = Array.prototype.slice.call(arguments);

      if (GATrack.isGA) {
        args.unshift('send');
      }

      GATrack.sendData.apply(null, args);
    },
    sendData: function sendData() {
      if (!GATrack.isEnabled) {
        return GATrack;
      } // eslint-disable-next-line prefer-rest-params


      var args = Array.prototype.slice.call(arguments);

      if (GATrack.isGA) {
        if (GATrack.trackerName) {
          args[0] = GATrack.trackerName + "." + args[0];
        }

        ga.apply(null, args);
      } else if (GATrack.isGTag) {
        gtag.apply(null, args);
      }
    },
    getData: function getData(element, options) {
      if (options === void 0) {
        options = {};
      }

      var href = element.dataset.gaTrackHref || element.getAttribute('href');
      var category = element.dataset.gaTrack || options.category || 'ga-track';
      var label = element.dataset.gaTrackLabel || options.label || href;
      var action = element.dataset.gaTrackAction || options.action || element.textContent.trim();
      return {
        href: href,
        category: category,
        label: label,
        action: action
      };
    },
    track: function track(element, options) {
      if (options === void 0) {
        options = {};
      }

      if (Array.isArray(element)) {
        element.forEach(function (data) {
          find(data.element).forEach(function (el) {
            GATrack.track(el, data);
          });
        });
        return;
      }

      if (typeof element.dataset.gaTrackInitialised !== 'undefined') {
        return;
      }

      element.dataset.gaTrackInitialised = 'true';
      options = aug({}, GATrack.defaults, options);
      this.log('tracking', element, options);
      on(element, 'click', function (event) {
        GATrack.onTrackedClick(element, event, options);
      });
    },
    onTrackedClick: function onTrackedClick(element, event, options) {
      var data = GATrack.getData(element, options);
      var target = element.getAttribute('target');
      var callback = null;

      if (element.dataset.gaTrackHref === 'false') {
        event.preventDefault();
      } else if (data.href && !event.metaKey && event.which === 1 && target !== '_blank') {
        event.preventDefault();

        callback = function callback() {
          window.location = data.href;
        };
      }

      GATrack.sendEvent(data.category, data.action, data.label, callback, options.timeout);
    },
    autotrack: function autotrack() {
      if (GATrack.autotracking === true) {
        return;
      }

      GATrack.autotracking = true;
      var selector = '[data-ga-track]';
      on(document.body, 'click', function (event) {
        var element = event.target;

        if (!matches(element, selector)) {
          element = closest(element, selector);
        }

        if (element) {
          GATrack.onTrackedClick(element, event, GATrack.defaults);
        }
      });
    },
    log: function log() {
      if (GATrack.debug) {
        var _console;

        for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
          args[_key] = arguments[_key];
        }

        (_console = console).log.apply(_console, ['GATRACK'].concat(args)); //eslint-disable-line no-console

      }
    },
    isNullOrEnforced: function isNullOrEnforced(provider) {
      return GATrack.force === null || GATrack.force === provider;
    },

    get isGAQ() {
      // eslint-disable-line no-underscore-dangle
      return typeof window._gaq !== 'undefined' && GATrack.isNullOrEnforced('gaq');
    },

    get isGTag() {
      return typeof window.gtag !== 'undefined' && GATrack.isNullOrEnforced('gtag');
    },

    get isGA() {
      return typeof window.ga !== 'undefined' && GATrack.isNullOrEnforced('ga');
    },

    get isEnabled() {
      return GATrack.isGA || GATrack.isGTag || GATrack.isGAQ;
    },

    debug: typeof window.localStorage === 'object' && window.localStorage.getItem('GATrackDebug'),
    prefix: null,
    trackerName: '',
    force: null,
    defaults: {
      timeout: 1000
    }
  };

  var outlineTracked = function outlineTracked() {
    return find('[data-ga-track-initialised], [data-ga-track]').map(function (el) {
      el.style.outline = 'red dotted 1px';
      return el;
    });
  };

  GATrack.debug = window.localStorage.getItem('GATrackDebug');
  var outline = window.localStorage.getItem('GATrackOutline');
  window.GAOutlineTracked = outlineTracked;
  ready(function () {
    if (!window._GATrack_) {
      window._GATrack_ = GATrack;

      if (typeof window.gaTrackerName_ !== 'undefined') {
        GATrack.trackerName = window.gaTrackerName_;
      }

      GATrack.autotrack();

      if (outline) {
        outlineTracked();
      }
    }
  });

  var scrollPercent = 0;
  var wHeight = 0;
  var dHeight = 0;
  var trackLength = 0;
  var location = document.location.toString();
  var hasScrolled = false;
  var cache = [25, 50, 75, 100];
  var Events = {
    Scroll: 'user:scroll'
  };

  function getDocHeight() {
    return Math.max(document.body.scrollHeight, document.documentElement.scrollHeight, document.body.offsetHeight, document.documentElement.offsetHeight, document.body.clientHeight, document.documentElement.clientHeight);
  }

  function computeSizes() {
    wHeight = window.innerHeight || (document.documentElement || document.body).clientHeight;
    dHeight = getDocHeight();
    trackLength = dHeight - wHeight;
  }

  function getScrolledAmount() {
    var scrollTop = window.pageYOffset || (document.documentElement || document.body.parentNode || document.body).scrollTop;
    return Math.floor(scrollTop / trackLength * 100);
  }

  function trackScrollPosition(amount, overriddenLabel) {
    var category = 'scroll';
    var action = location;
    var label = overriddenLabel || "Scrolled " + amount + "%";

    if (GATrack.isGTag) {
      // Gtag check needs to go before since gtag creates a ga variable
      var payload = {
        event_category: category,
        event_label: label,
        value: amount
      };
      GATrack.sendData('event', action, payload);
    } else {
      GATrack.sendData('send', 'event', category, action, label, amount);
    }

    fire(document.body, Events.Scroll, {
      detail: {
        amount: amount
      }
    });
  }

  function scrollCheck() {
    if (!cache.length) {
      return;
    }

    scrollPercent = getScrolledAmount();

    if (!hasScrolled && scrollPercent > 0) {
      hasScrolled = true;
      GATrack.sendEvent('scroll', location, 'Scrolled');
    }

    var events = cache.filter(function (a) {
      return a <= scrollPercent;
    });
    events.forEach(function (e) {
      trackScrollPosition(e);
      cache.splice(cache.indexOf(e), 1);
    });
  }

  ready(function () {
    if (window.disableGATrackScroll !== true) {
      computeSizes();
      index.on(scrollCheck);
      window.addEventListener('resize', computeSizes, false);
    }
  });

  exports.Events = Events;
  exports.trackScrollPosition = trackScrollPosition;

  return exports;

}({}));
//# sourceMappingURL=ga-track-scroll.bundle.js.map
