import ScrollBus from 'scroll-bus';
import GATrack from 'ga-track';
import { fire, ready } from 'domassist';

var scrollPercent = 0;
var wHeight = 0;
var dHeight = 0;
var trackLength = 0;
var location = document.location.toString();
var hasScrolled = false;
var cache = [25, 50, 75, 100];
var Events = {
  Scroll: 'user:scroll'
};

function getDocHeight() {
  return Math.max(document.body.scrollHeight, document.documentElement.scrollHeight, document.body.offsetHeight, document.documentElement.offsetHeight, document.body.clientHeight, document.documentElement.clientHeight);
}

function computeSizes() {
  wHeight = window.innerHeight || (document.documentElement || document.body).clientHeight;
  dHeight = getDocHeight();
  trackLength = dHeight - wHeight;
}

function getScrolledAmount() {
  var scrollTop = window.pageYOffset || (document.documentElement || document.body.parentNode || document.body).scrollTop;
  return Math.floor(scrollTop / trackLength * 100);
}

function trackScrollPosition(amount, overriddenLabel) {
  var category = 'scroll';
  var action = location;
  var label = overriddenLabel || "Scrolled " + amount + "%";

  if (GATrack.isGTag) {
    // Gtag check needs to go before since gtag creates a ga variable
    var payload = {
      event_category: category,
      event_label: label,
      value: amount
    };
    GATrack.sendData('event', action, payload);
  } else {
    GATrack.sendData('send', 'event', category, action, label, amount);
  }

  fire(document.body, Events.Scroll, {
    detail: {
      amount: amount
    }
  });
}

function scrollCheck() {
  if (!cache.length) {
    return;
  }

  scrollPercent = getScrolledAmount();

  if (!hasScrolled && scrollPercent > 0) {
    hasScrolled = true;
    GATrack.sendEvent('scroll', location, 'Scrolled');
  }

  var events = cache.filter(function (a) {
    return a <= scrollPercent;
  });
  events.forEach(function (e) {
    trackScrollPosition(e);
    cache.splice(cache.indexOf(e), 1);
  });
}

ready(function () {
  if (window.disableGATrackScroll !== true) {
    computeSizes();
    ScrollBus.on(scrollCheck);
    window.addEventListener('resize', computeSizes, false);
  }
});

export { Events, trackScrollPosition };
//# sourceMappingURL=ga-track-scroll.js.map
